@@ -102,18 +102,20 @@ $ds->define_node({ path => 'css',
 
 $ds->define_node(
     { path => 'single',
-      title => 'Single States',
-      usage => [ { params => 'state=wi', format => 'json' },
-      		 { params => 'state=tx&show=hist&header=no', format => 'txt' } ],
+      title => 'Single state',
+      place => 1,
+      usage => [ "single.json?state=wi",
+		 "single.txt?state=tx&show=hist&header=no" ],
       output => 'basic',
       optional_output => 'extra',
       role => 'PopulationData',
       method => 'single' },
 	"Returns information about a single U.S. state.",
     { path => 'list',
-      title => 'Multiple States',
-      usage => [ { params => 'region=ne&show=total', format => 'json' },
-		 { params => 'region=we&show=hist,total&count&datainfo', format => 'txt' } ],
+      title => 'Multiple states',
+      place => 2,
+      usage => [ "list.json?region=ne&show=total",
+		 "list.txt?region=we&show=hist,total&count&datainfo" ],
       output => 'basic',
       optional_output => 'extra',
       role => 'PopulationData',
@@ -121,7 +123,8 @@ $ds->define_node(
 	"Returns information about all of the states matching specified criteria.",
     { path => 'regions',
       title => 'Regions',
-      usage => { format => 'json' },
+      place => 3,
+      usage => "regions.txt",
       output => 'regions',
       role => 'PopulationData',
       method => 'regions' },
